package FDN::Etebac::Sepa;

use strict;
use warnings;
use base 'Exporter';

use Carp;
use DateTime;
use Template;
use FDN::Common::Modules;
use FDN::Common::Config;
use FDN::Common::CharTable;

our @EXPORT = qw (
	rib_to_bic
	rib_to_iban

	dd_init_file
	dd_add_line
	dd_make_file
);



my $letters;
BEGIN {
	$letters = {};
	my $i;
	foreach $i (0..25) {
		my $l = chr(ord('a')+$i);
		$letters->{$l} = $i+10;
	}
}

sub __char_to_num($) {
	my ( $iban ) = @_;
	$iban = lc($iban);
	$iban =~ s/([a-z])/$letters->{$1}/g;
	return $iban;
}

my $banques;
sub __read_banques {
	my $file = '/usr/share/fdn/data/banques.csv';
	if ( ! -f $file ) {
		confess "File not found: $file (list of banks BIC for France)";
	}
	$banques = {};
	my $line;
	open FILE, '<', $file or die "Cannot read $file: $@";
	while ( $line = <FILE> ) {
		next if $line =~ /^#/;
		next if $line !~ /^FR;([^;]*);([^;]*);(\d+);([^;]*)/;
		my ( $banque, $bic, $code, $ville ) = ( $1, $2, $3, $4 );
		if ( exists $banques->{$code} && $banques->{$code} ne $bic ) {
			confess "J'ai une deuxieme banque avec le meme code guichet: $code in $line.";
		}
		$banques->{$code} = $bic;
	}
	close FILE;
}

sub rib_to_iban {
	my ( $banque, $guichet, $compte, $clefrib ) = @_;
	conduse('Math::BigInt');

	my $clef = $banque.$guichet.$compte.$clefrib.'FR00';
	$clef = __char_to_num($clef);
	$clef = Math::BigInt->new($clef);
	$clef = $clef->bmod(97);
	$clef = 98 - $clef;
	if ( $clef < 10 ) {
		$clef = '0'.$clef;
	}
	return 'FR'.$clef.$banque.$guichet.$compte.$clefrib;
}

sub rib_to_bic {
	my ( $banque, $guichet, $compte, $clef ) = @_;
	if ( ! $banques ) {
		__read_banques();
	}
	if ( ! exists $banques->{$banque} ) {
		return undef;
	}
	return $banques->{$banque};
}


sub dd_init_file($$) {
	my ($name, $date) = @_;

	my $info;

	my $cfg = $FDN::Common::Config::CONFIG_ROOT.'sepa/'.$name.'.cfg';
	if ( -e $cfg ) {
		$info = Config_Section('sepa/'.$name.'.cfg','banque');
	}
	if ( ! defined $info ) {
		confess "Il me faut un fichier de configuration '$cfg' avec une section [banque]";
	}
	if ( ! defined $info->{'raison-soc'} || ! defined $info->{'ics'} || ! defined $info->{'bic'} || ! defined $info->{'iban'} ) {
		confess "Il me manque une info dans '/etc/fdn/sepa/$name.cfg'/[banque]. Je veux raison-soc, ics, bic et iban.";
	}

	my $now = DateTime->now();
	my $id = $now->ymd().'-'.$now->hms('').'-001'; # Pas plus d'un fichier produit par seconde
	my $dt = $now->ymd('-').'T'.$now->hms(':');

	my $dd = {};
	$dd->{'file'} = {
		'id'       => $id,
		'datetime' => $dt,
		'nb_tx'    => 0,
		'total'    => 0
	};
	$dd->{'creditor'} = {
		'name' => $info->{'raison-soc'},
		'iban' => $info->{'iban'},
		'bic'  => $info->{'bic'},
		'ics'  => $info->{'ics'}
	};
	$dd->{'__exec_date'} = $date;
	$dd->{'__now'} = $now;
	if ( $info->{'no-accent'} ) {
		$dd->{'__no_accent'} = 1;
	} else {
		$dd->{'__no_accent'} = 0;
	}
	return $dd;
}

sub dd_add_line($$$) {
	my ( $dd, $type, $prllig ) = @_;

	if ( $type !~ /^(FRST|RCUR|FNAL|OOFF)$/ ) {
		confess "Le type '$type' est inconnu.";
	}
	if ( exists $prllig->{'PRLLIG_ID'} ) {
		confess "Moi, je fais des prelevement SEPA, ca c'est une ligne de prel. Etebac";
	}
	if ( ! exists $prllig->{'PRLLIGSEPA_BIC'} || ! exists $prllig->{'PRLLIGSEPA_IBAN'} ||
	     ! exists $prllig->{'PRLLIGSEPA_MANDAT'} || ! exists $prllig->{'PRLLIGSEPA_DATEMANDAT'} ) {
		confess "Le prllig n'est pas valide... man des infos sur le compte... Pas migr en SEPA ?";
	}
	if ( ! exists $prllig->{'PRLLIGSEPA_MONTANT'} || $prllig->{'PRLLIGSEPA_MONTANT'} !~ /^\d+$/ ) {
		confess "La ligne de prelevement n'a pas de montant, ce n'est pas normal";
	}

	my $mnt = $prllig->{'PRLLIGSEPA_MONTANT'};
	if ( $mnt < 10 ) {
		$mnt = '0.0'.$mnt;
	} elsif ( $mnt < 100 ) {
		$mnt = '0.'.$mnt;
	} else {
		$mnt =~ s/(\d\d)$/.$1/;
	}

	my $date = $prllig->{'PRLLIGSEPA_DATEMANDAT'};
	$date =~ s/(T| )\d\d:\d\d:\d\d$//;

	my $label     = $prllig->{'PRLLIGSEPA_LIBELLE'};
	my $titulaire = $prllig->{'PRLLIGSEPA_TITULAIRE'};
	if ( $dd->{'__no_accent'} ) {
		$label     = cleanup_chars_as_table($label, $iso8859_15_to_rfc822);
		$titulaire = cleanup_chars_as_table($titulaire, $iso8859_15_to_rfc822); 
	}

	$dd->{$type} //= [];
	my $trans = {};
	$trans->{'id'} = $prllig->{'PRLLIGSEPA_ID'};
	$trans->{'amount'} = $mnt;
	$trans->{'label'}  = $label;
	$trans->{'currency'} = 'EUR';
	$trans->{'mandate_id'} = $prllig->{'PRLLIGSEPA_MANDAT'};
	$trans->{'mandate_date'} = $date;
	$trans->{'name'} = $titulaire;
	$trans->{'iban'} = $prllig->{'PRLLIGSEPA_IBAN'};
	$trans->{'bic'}  = $prllig->{'PRLLIGSEPA_BIC'};
	push @{$dd->{$type}}, $trans;

	$dd->{'file'}->{'nb_tx'}++;
	$dd->{'file'}->{'total'} += $mnt;
	return $dd;
}

sub dd_make_file($$) {
	my ( $dd, $filename ) = @_;

	my $vars;
	$vars->{'file'} = $dd->{'file'};
	$vars->{'creditor'} = $dd->{'creditor'};
	$vars->{'infos'} = [];

	my $count = 0;
	for my $type ( 'FRST', 'RCUR', 'FNAL', 'OOFF' ) {
		next if ! $dd->{$type};
		my $T = {};
		$T->{'type'} = $type;
		$T->{'date'} = $dd->{'__exec_date'};
		$T->{'id'}   = $dd->{'__now'}->ymd('-').'-'.$type.'-001'; # Pas plus d'un fichier produit par jour
		$T->{'transactions'} = [];
		my $total = 0;
		for my $trans ( @{$dd->{$type}} ) {
			my $tmp = { %$trans };
			$tmp->{'id'} = $dd->{'__exec_date'}.'-'.$trans->{'id'};
			$total += $tmp->{'amount'};
			push @{$T->{'transactions'}}, $tmp;
		}
		$T->{'total'} = $total;
		push @{$vars->{'infos'}}, $T;
	}

	#if ( -e $filename ) {
	#	confess "Le fichier '$filename' existe deja. J'arrete.";
	#}
	open (my $fh, '>', $filename) or confess "Failed to open '$filename'. Je ne peux pas ecrire. J'arrete.";

	my $tt = Template->new({ INCLUDE_PATH => [ '/usr/share/fdn/templates/sepa' ] });
	if ( ! $tt->process('direct_debit.tt',$vars, $fh) ) {
		confess "Failed tt rendering: $@\n\n";
	}
	close $fh;
}


1;
